@@ -44,10 +44,19 @@
 #endif
 #include <gdk/gdkkeysyms.h>
 
+//xml
+#include <stdio.h>
+#include <stdlib.h>
+#include <libxml/xmlmemory.h>
+#include <libxml/parser.h>
+
+#define LINE_LIMIT 500
+
 /* Privates: */
 void safeReplaceString(char** oldstr, const gchar *newstr, MainView *mainview);
 void maybe_save_file(MainView *mainview);
 void save_mod_file(MainView *mainview);
+void setCurrentGroup(MainView *mainview, gchar *group);
 
 /* activate grid */
 void callback_activate_treeView(GtkTreeView *treeView, GtkTreePath *path, GtkTreeViewColumn *column, MainView *mainview) {
@@ -58,22 +67,22 @@ void callback_activate_treeView(GtkTreeView *treeView, GtkTreePath *path, GtkTre
 void callback_hildon_activate_treeView(GtkTreeView *treeView, GtkTreePath *path, MainView *mainview) {
 	GtkTreeIter iter;
 	rec *onrec=NULL;
+	gchar *label=NULL;
+	gboolean groupExpand=FALSE;
 	g_assert(mainview != NULL && mainview->data != NULL );
 	if ( !gtk_tree_model_get_iter(GTK_TREE_MODEL(mainview->store), &iter, path) ) {
 		return;
 	}
-	gtk_tree_model_get(GTK_TREE_MODEL(mainview->store), &iter,
-	    DATA_COLUMN, &onrec,
-	    -1);
-	/*
-	onrec=g_object_get_data (G_OBJECT(&iter), "r");
-	*/
+	gtk_tree_model_get(GTK_TREE_MODEL(mainview->store), &iter, LABEL_COLUMN, &label, DATA_COLUMN, &onrec, GROUP_COLUMN, &groupExpand, -1);
 	if (onrec!=NULL) {
 		show_rec(mainview, onrec, FALSE);
-	} else if (gtk_tree_view_collapse_row(treeView, path)) {
-		gtk_tree_view_expand_row(treeView, path, FALSE);
+		setCurrentGroup(mainview, onrec->group);
 	} else {
-		gtk_tree_view_collapse_row(treeView, path);
+		setCurrentGroup(mainview, label);
+		if (groupExpand) {
+			gtk_tree_view_collapse_row (treeView, path);
+		}
+		gtk_tree_store_set(mainview->store, &iter, GROUP_COLUMN, !groupExpand, -1);
 	}
 }
 
@@ -151,7 +160,7 @@ void callback_rec_response(GtkDialog *dialog, gint response, MainView *mainview)
 			gtk_clipboard_set_text(mainview->clipboard, rec->password, -1);
 			break;
 		case 4:
-			if ((recwindow->editable) && (CONFRESP_YES==interface_yes_no_note(mainview, "Confirm Delete?"))) {
+			if ((recwindow->editable) && (CONFRESP_YES==interface_yes_no_note(mainview, _("Confirm Delete ?")))) {
 				unshow_rec(mainview);
 				delete_rec(rec);
 				populate_treeView(mainview); /* redraw grid */
@@ -301,7 +310,7 @@ gboolean callback_tag_event(GtkTextTag *tag, GObject *object, GdkEvent *event, G
 			}
 		}
 		text = gtk_text_iter_get_text(&start, &end);
-		g_print("text associ au tag %s\n", text);
+//		g_print("text associ au tag %s\n", text);
 		launch_browser(text, mainView);
 	}
 	return FALSE;
@@ -310,6 +319,11 @@ gboolean callback_tag_event(GtkTextTag *tag, GObject *object, GdkEvent *event, G
 /* Callback for hardware D-BUS events */
 void hw_event_handler(osso_hw_state_t *state, MainView *mainview) {
  	g_assert(mainview != NULL && mainview->data != NULL );
+ 	
+ 	if ((state->system_inactivity_ind) && isRecBeingEdited(mainview)) {
+		// Bug 9250
+		return;
+	}
 
 	if ((state->shutdown_ind)	/* system shutdown */
 			|| (state->system_inactivity_ind)) {	/* system sleeping */
@@ -320,7 +334,10 @@ void hw_event_handler(osso_hw_state_t *state, MainView *mainview) {
 	if (state->memory_low_ind) {	/* memory low... */
 		unshow_rec(mainview);	/* close the rec window... changes not saved... */
 	}
-    save_mod_file(mainview);
+	if (!isRecBeingEdited(mainview)) {
+		// Bug 9250
+		save_mod_file(mainview);
+	}
 
     /*
 	if (state->save_unsaved_data_ind && mainview->file_edited) {	/ * save unsaved data... * /
@@ -347,7 +364,7 @@ gboolean exit_event_handler( GtkWidget *widget, GdkEvent  *event, MainView *main
 void maybe_save_file(MainView *mainview) {
 /* save changes if file changed, with requester*/
     if (mainview->pass && mainview->file_edited) {
-        if (CONFRESP_YES==interface_yes_no_note(mainview, "Save Changes?")) { 
+        if (CONFRESP_YES==interface_yes_no_note(mainview, _("Save Changes ?"))) { 
             if (!mainview->file_name) {  /* should never happen because file name set on open */
                 mainview->file_name = interface_file_chooser(mainview, GTK_FILE_CHOOSER_ACTION_SAVE);
             }
@@ -367,9 +384,14 @@ void save_mod_file(MainView *mainview) {
 }
 
 gchar * getCurrentGroup(MainView *mainview ) {
-	//TODO get the current Group from the selected one
-	gchar *group = g_strdup("");
-	return group;
+	return mainview->currentGroup;
+}
+
+void setCurrentGroup(MainView *mainview, gchar *group) {
+	if (group != NULL && group[0] != '\0' && strcmp(group, _(DEFAULT_GROUP)) != 0) {
+		g_free(mainview->currentGroup);
+		mainview->currentGroup = g_strdup(group);
+	}
 }
 
 /* add entry */
@@ -386,7 +408,7 @@ void callback_add_entry ( GtkAction * action, MainView *mainview ) {
 void callback_change_password(GtkAction * action, MainView *mainview) {
 	char *newpass=NULL;
 	g_assert(mainview != NULL && mainview->data != NULL );
-	newpass=get_password(mainview, "Change Password", PWD_DLG_CHANGE);
+	newpass=get_password(mainview, _("Change Password"), PWD_DLG_CHANGE);
 	if (newpass) {
 		mainview->pass=newpass;
 		mainview->file_edited = TRUE;
@@ -400,7 +422,7 @@ void callback_file_new(GtkAction * action, MainView *mainview) {
 	/* maybe_save_file(mainview); */
 	save_mod_file(mainview);
 	close_file(mainview);
-	newpass=get_password(mainview, "New Password", PWD_DLG_NEW);
+	newpass=get_password(mainview, _("New Password"), PWD_DLG_NEW);
 	if (newpass) {
 		mainview->pass=newpass;
 //		add_new_record("new", "Group");
@@ -418,7 +440,7 @@ void callback_file_open(GtkAction * action, MainView *mainview) {
 	g_assert(mainview != NULL && mainview->data != NULL );
  	/* maybe_save_file(mainview); */
     save_mod_file(mainview);
-	mainview->file_name = interface_file_chooser ( mainview, GTK_FILE_CHOOSER_ACTION_OPEN );
+	mainview->file_name = interface_file_chooser ( mainview, GTK_FILE_CHOOSER_ACTION_OPEN);
 	read_pwsafe_file(mainview);
 }
 
@@ -505,6 +527,23 @@ void callback_filter(GtkWidget *widget, MainView *mainview ) {
     populate_treeView(mainview);
 }
 
+
+gboolean callback_tree_key_pressed(GtkTreeView *treeView, GdkEventKey *event, MainView *mainview) {
+	gchar *utf8String = NULL;
+	guint32 ucs4Char = gdk_keyval_to_unicode(event->keyval);
+	if (g_unichar_isalnum(ucs4Char)) {
+		utf8String = g_strnfill(10, '\0');
+		if (g_unichar_to_utf8(ucs4Char, utf8String) > 0) {
+			gtk_entry_set_text(GTK_ENTRY(mainview->filter_tb), utf8String);
+			callback_filter(mainview->filter_tb, mainview);
+			gtk_widget_grab_focus(mainview->filter_tb);
+			gtk_editable_set_position(GTK_EDITABLE(mainview->filter_tb), -1);
+		}
+	}
+	g_free(utf8String);
+	return FALSE;
+}
+
 gint getEntryLength(GtkEntry *entry) {
 #if MAEMO_VERSION_MAJOR < 5
 	gchar *text;
@@ -621,12 +660,13 @@ void callback_pwdialog_button(GtkWidget *widget, PasswordUI *pwUi) {
 
 /* export */
 void callback_export ( GtkAction * action, MainView *mainview ) {
-    char *fname;
+    gchar *fname;
     
     fname = interface_file_chooser(mainview, GTK_FILE_CHOOSER_ACTION_SAVE);
     if (fname != NULL) {
         FILE *outf;
         outf = fopen(fname,"w");
+        g_free(fname);
         if (outf != (FILE *) NULL) {
             GList *onrec=NULL;
      
@@ -649,6 +689,272 @@ void callback_export ( GtkAction * action, MainView *mainview ) {
     }
 }
 
+gint importXmlKeePassX(MainView *mainview, xmlDocPtr doc)  {
+	gint imported = 0;
+	xmlNodePtr cur;
+	xmlNodePtr curGroup;
+	xmlNodePtr curEntry;
+	xmlChar *groupName = NULL;
+	xmlChar *entryName = NULL;
+	xmlChar *userName = NULL;
+	xmlChar *password = NULL;
+	xmlChar *notes2 = NULL;
+	xmlChar *notes = NULL;
+	xmlChar *notes1 = NULL;
+	int strLen = 0;
+	
+	rec * record;
+
+	cur = xmlDocGetRootElement(doc);
+
+	if (cur == NULL) {
+		fprintf(stderr,"empty document\n");
+	} else {
+		if (xmlStrcasecmp(cur->name, (const xmlChar *) "database")) {
+			msgbox(mainview,_("document of the wrong type, root node != database"));
+		} else {
+			//Group
+			curGroup = cur->xmlChildrenNode;
+			while (curGroup != NULL) {
+				if ((!xmlStrcasecmp(curGroup->name, (const xmlChar *)"group"))){
+					cur = curGroup->xmlChildrenNode;
+					while (cur != NULL) {
+						if ((!xmlStrcasecmp(cur->name, (const xmlChar *)"title"))){
+							if (groupName != NULL) {
+								xmlFree(groupName);
+							}
+							groupName = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+						} else if ((!xmlStrcasecmp(cur->name, (const xmlChar *)"entry"))){
+							// loop on entry
+							curEntry = cur->xmlChildrenNode;
+							while (curEntry != NULL) {
+								if ((!xmlStrcasecmp(curEntry->name, (const xmlChar *)"title"))){
+									entryName = xmlNodeListGetString(doc, curEntry->xmlChildrenNode, 1);
+								} else if ((!xmlStrcasecmp(curEntry->name, (const xmlChar *)"username"))){
+									userName = xmlNodeListGetString(doc, curEntry->xmlChildrenNode, 1);
+								} else if ((!xmlStrcasecmp(curEntry->name, (const xmlChar *)"password"))){
+									password = xmlNodeListGetString(doc, curEntry->xmlChildrenNode, 1);
+								} else if ((!xmlStrcasecmp(curEntry->name, (const xmlChar *)"url")) || (!xmlStrcasecmp(curEntry->name, (const xmlChar *)"comment"))){
+									if (notes != NULL) {
+										notes2 = xmlNodeListGetString(doc, curEntry->xmlChildrenNode, 1);
+										if (notes2 != NULL) {
+											strLen = xmlStrlen(notes2);
+											if (strLen > LINE_LIMIT) {
+												strLen = LINE_LIMIT;
+											}
+											if (strLen > 0) {
+												notes1 = xmlStrncatNew(notes, "\r\n", 2);
+												xmlFree(notes);
+												notes = xmlStrncatNew(notes1, notes2, strLen);
+												xmlFree(notes1);
+												xmlFree(notes2);
+											}
+										}
+									} else {
+										notes = xmlNodeListGetString(doc, curEntry->xmlChildrenNode, 1);
+										if ((notes != NULL) && (xmlStrlen(notes) == 0)) {
+											xmlFree(notes);
+											notes = NULL;
+										}
+									}
+								}
+								curEntry = curEntry->next;
+							}
+							if (entryName != NULL || userName != NULL) {
+								if (groupName == NULL) {
+									groupName = xmlCharStrdup(_("Group"));
+								}
+								imported ++;
+								record = add_new_record((entryName == NULL)?userName:entryName, groupName);
+								record->user = g_strdup(userName);
+								record->password = g_strdup(password);
+								record->notes = g_strdup(notes);
+							}
+							if (entryName != NULL) {
+								xmlFree(entryName);
+								entryName = NULL;
+							}
+							if (userName != NULL) {
+								xmlFree(userName);
+								userName = NULL;
+							}
+							if (password != NULL) {
+								xmlFree(password);
+								password = NULL;
+							}
+							if (notes != NULL) {
+								xmlFree(notes);
+								notes = NULL;
+							}
+						}
+						cur = cur->next;
+					}
+				}
+				curGroup = curGroup->next;
+			}
+			if (groupName != NULL) {
+				xmlFree(groupName);
+				groupName = NULL;
+			}
+		}
+	}
+	return imported;
+}
+
+gint importXmlFPM2(MainView *mainview, xmlDocPtr doc)  {
+	gint imported = 0;
+	xmlNodePtr cur;
+	xmlNodePtr curGroup;
+	xmlNodePtr curEntry;
+	xmlChar *groupName = NULL;
+	xmlChar *entryName = NULL;
+	xmlChar *userName = NULL;
+	xmlChar *password = NULL;
+	xmlChar *notes2 = NULL;
+	xmlChar *notes = NULL;
+	xmlChar *notes1 = NULL;
+	int strLen = 0;
+	
+	rec * record;
+
+	cur = xmlDocGetRootElement(doc);
+
+	if (cur == NULL) {
+		fprintf(stderr,"empty document\n");
+	} else {
+		if (xmlStrcasecmp(cur->name, (const xmlChar *) "FPM")) {
+			msgbox(mainview,_("document of the wrong type, root node != FPM"));
+		} else {
+			//List
+			cur = cur->xmlChildrenNode;
+			while (cur != NULL) {
+				if ((!xmlStrcasecmp(cur->name, (const xmlChar *)"PasswordList"))){
+					cur = cur->xmlChildrenNode;
+					while (cur != NULL) {
+						if ((!xmlStrcasecmp(cur->name, (const xmlChar *)"PasswordItem"))){
+							// loop on entry
+							curEntry = cur->xmlChildrenNode;
+							while (curEntry != NULL) {
+								if ((!xmlStrcasecmp(curEntry->name, (const xmlChar *)"title"))){
+									entryName = xmlNodeListGetString(doc, curEntry->xmlChildrenNode, 1);
+								} else if ((!xmlStrcasecmp(curEntry->name, (const xmlChar *)"user"))){
+									userName = xmlNodeListGetString(doc, curEntry->xmlChildrenNode, 1);
+								} else if ((!xmlStrcasecmp(curEntry->name, (const xmlChar *)"password"))){
+									password = xmlNodeListGetString(doc, curEntry->xmlChildrenNode, 1);
+								} else if ((!xmlStrcasecmp(curEntry->name, (const xmlChar *)"category"))){
+									groupName = xmlNodeListGetString(doc, curEntry->xmlChildrenNode, 1);
+								} else if ((!xmlStrcasecmp(curEntry->name, (const xmlChar *)"url")) || (!xmlStrcasecmp(curEntry->name, (const xmlChar *)"notes"))){
+									if (notes != NULL) {
+										notes2 = xmlNodeListGetString(doc, curEntry->xmlChildrenNode, 1);
+										if (notes2 != NULL) {
+											strLen = xmlStrlen(notes2);
+											if (strLen > LINE_LIMIT) {
+												strLen = LINE_LIMIT;
+											}
+											if (strLen > 0) {
+												notes1 = xmlStrncatNew(notes, "\r\n", 2);
+												xmlFree(notes);
+												notes = xmlStrncatNew(notes1, notes2, strLen);
+												xmlFree(notes1);
+												xmlFree(notes2);
+											}
+										}
+									} else {
+										notes = xmlNodeListGetString(doc, curEntry->xmlChildrenNode, 1);
+										if ((notes != NULL) && (xmlStrlen(notes) == 0)) {
+											xmlFree(notes);
+											notes = NULL;
+										}
+									}
+								}
+								curEntry = curEntry->next;
+							}
+							if (entryName != NULL || userName != NULL) {
+								if (groupName == NULL) {
+									groupName = xmlCharStrdup(_("Group"));
+								}
+								imported ++;
+								record = add_new_record((entryName == NULL)?userName:entryName, groupName);
+								record->user = g_strdup(userName);
+								record->password = g_strdup(password);
+								record->notes = g_strdup(notes);
+							}
+							xmlFree(groupName);
+							if (entryName != NULL) {
+								xmlFree(entryName);
+								entryName = NULL;
+							}
+							if (userName != NULL) {
+								xmlFree(userName);
+								userName = NULL;
+							}
+							if (password != NULL) {
+								xmlFree(password);
+								password = NULL;
+							}
+							if (notes != NULL) {
+								xmlFree(notes);
+								notes = NULL;
+							}
+						}
+						cur = cur->next;
+					}
+				}
+				if (cur != NULL) {
+					cur = cur->next;
+				}
+			}
+		}
+	}
+	return imported;
+}
+
+void callback_import_xml(GtkAction * action, MainView *mainview ) {
+	xmlDocPtr doc;
+	gchar *notes = NULL;
+	gchar *fname;
+	
+	gint imported = 0;
+	gint xmlFormat = PWSAFE_KEEPASSX;
+	
+	xmlFormat = interface_getXmlFormat(mainview, xmlFormat);
+	if (xmlFormat < 0) {
+		return;
+	}
+	if (xmlFormat >= PWSAFE_NB_XML_IMPORT) {
+		msgbox(mainview, _("Import XML Format not yet supported!"));
+		return;
+	}
+	
+    fname = interface_file_chooser(mainview, GTK_FILE_CHOOSER_ACTION_OPEN);
+    if (fname != NULL) {
+		doc = xmlParseFile(fname);
+	
+		if (doc == NULL ) {
+			msgbox(mainview,_("Document not parsed successfully."));
+		} else {
+			if (xmlFormat == PWSAFE_KEEPASSX) {
+				imported = importXmlKeePassX(mainview, doc);
+			} else if (xmlFormat == PWSAFE_FPM2) {
+				imported = importXmlFPM2(mainview, doc);
+			} else {
+				msgbox(mainview,_("Import XML Format not yet supported!"));
+			}
+			xmlFreeDoc(doc);
+		}
+		g_free(fname);
+		if (imported > 0) {
+			populate_treeView(mainview);
+			notes = g_strdup_printf(_("%d items imported."), imported);
+			msgbox(mainview, notes);
+			g_free(notes);
+		} else {
+			msgbox(mainview, _("No item imported."));
+		}
+	}
+}
+
+	
 DBusHandlerResult dbus_handle_mce_message(DBusConnection *con, DBusMessage *msg, MainView *mainView ) {
 	DBusMessageIter iter;
 	dbus_bool_t value;
@@ -664,3 +970,5 @@ DBusHandlerResult dbus_handle_mce_message(DBusConnection *con, DBusMessage *msg,
 	}
 	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
+
+
